Exception: Grifter::RequestException
- Defined in:
- lib/grifter/http_service.rb
Instance Method Summary collapse
- #body ⇒ Object
-
#code ⇒ Object
shortcut methods.
-
#initialize(request, response) ⇒ RequestException
constructor
A new instance of RequestException.
-
#to_s ⇒ Object
this makes good info show up in rspec reports.
Constructor Details
#initialize(request, response) ⇒ RequestException
Returns a new instance of RequestException.
151 152 153 |
# File 'lib/grifter/http_service.rb', line 151 def initialize(request, response) @request, @response = request, response end |
Instance Method Details
#body ⇒ Object
165 166 167 |
# File 'lib/grifter/http_service.rb', line 165 def body @response.body end |
#code ⇒ Object
shortcut methods
161 162 163 |
# File 'lib/grifter/http_service.rb', line 161 def code @response.code end |
#to_s ⇒ Object
this makes good info show up in rspec reports
156 157 158 |
# File 'lib/grifter/http_service.rb', line 156 def to_s "#{self.class}\nResponseCode: #{@response.code}\nResponseBody:\n#{@response.body}" end |