Exception: Grifter::RequestException
- Defined in:
- lib/grifter/http_service.rb
Instance Method Summary collapse
- #body ⇒ Object
-
#code ⇒ Object
shortcut methods.
-
#initialize(request, response) ⇒ RequestException
constructor
A new instance of RequestException.
-
#to_s ⇒ Object
this makes good info show up in rspec reports.
Constructor Details
#initialize(request, response) ⇒ RequestException
123 124 125 |
# File 'lib/grifter/http_service.rb', line 123 def initialize(request, response) @request, @response = request, response end |
Instance Method Details
#body ⇒ Object
137 138 139 |
# File 'lib/grifter/http_service.rb', line 137 def body @response.body end |
#code ⇒ Object
shortcut methods
133 134 135 |
# File 'lib/grifter/http_service.rb', line 133 def code @response.code end |
#to_s ⇒ Object
this makes good info show up in rspec reports
128 129 130 |
# File 'lib/grifter/http_service.rb', line 128 def to_s "#{self.class}\nResponseCode: #{@response.code}\nResponseBody:\n#{@response.body}" end |